<?php

namespace JUM\PaymentProviders;

use XF\AddOn\{AbstractSetup, StepRunnerInstallTrait, StepRunnerUpgradeTrait, StepRunnerUninstallTrait};

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	public function checkRequirements(&$errors = [], &$warnings = [])
	{
		$em = $this->app()->em();
		$existAddon = null;

		$installProviders = array_merge($this->getProviders(), ['qiwiapi' => '']);
		$findProviders = $em->findByIds('XF:PaymentProvider', array_keys($installProviders));
		$existProviders = $findProviders->pluckNamed('addon_id', 'provider_id');

		foreach ($existProviders as $providerId => $addonId)
		{
			if ($addonId != $this->addOn->getAddOnId())
			{
				$errors[] = 'A function conflict was detected. Please remove the Add-on first: ' . $addonId;
			}
		}
	}

	public function installStep1()
	{
		foreach ($this->getProviders() as $id => $class)
		{
			$this->db()->insert('xf_payment_provider',
				[
				'provider_id' => $id,
				'provider_class' => 'JUM\\PaymentProviders:' . $class,
				'addon_id' => $this->addOn->getAddOnId()
				], 'provider_id');
		}
	}

	public function upgrade2010070Step1()
	{
		$this->uninstall();
		$this->install();
	}


	public function upgrade2010070Step2()
	{
		$path = $this->addOn->getAddOnDirectory() . '/Payment/';
		$QiwiApi = $path . 'QiwiApi.php';
		$WalletOne = $path . 'WalletOne.php';

		if (file_exists($QiwiApi)) unlink($QiwiApi);
		if (file_exists($WalletOne)) unlink($WalletOne);
	}

	public function upgrade2020170Step1()
	{
		$newClass = 'JUM\\PaymentProviders:YooMoney';
		$oldId = 'yandexmoney';
		$newId = 'yoomoney';

		$this->db()->update('xf_payment_provider',
			['provider_id' => $newId,
				'provider_class' => $newClass],
			'provider_id = ?', $oldId
		);
	}

	public function upgrade2020170Step2()
	{
		$oldId = 'yandexmoney';
		$newId = 'yoomoney';

		$this->db()->update('xf_payment_profile',
			['provider_id' => $newId],
			'provider_id = ?', $oldId
		);
	}

	public function upgrade2020170Step3()
	{
		$path = $this->addOn->getAddOnDirectory() . '/Payment/';
		$YandexMoney = $path . 'YandexMoney.php';

		if (file_exists($YandexMoney)) unlink($YandexMoney);
	}

	public function uninstallStep1()
	{
		$this->db()->delete('xf_payment_provider', 'addon_id LIKE ?', $this->addOn->getAddOnId());
	}

	/**
	 * @param array $stateChanges
	 */
	public function postInstall(array &$stateChanges)
	{
		if ($this->applyDefaultPermissions())
		{
			$this->app->jobManager()->enqueueUnique(
				'permissionRebuild',
				'XF:PermissionRebuild',
				[],
				false
			);
		}
	}

	/**
	 * @param $previousVersion
	 * @param array $stateChanges
	 */
	public function postUpgrade($previousVersion, array &$stateChanges)
	{
		if ($this->applyDefaultPermissions($previousVersion))
		{
			$this->app->jobManager()->enqueueUnique(
				'permissionRebuild',
				'XF:PermissionRebuild',
				[],
				false
			);
		}
	}

	/**
	 * @param null $previousVersion
	 * @return bool
	 */
	protected function applyDefaultPermissions($previousVersion = null)
	{
		$applied = false;

		if (!$previousVersion || $previousVersion < 2010070)
		{
			$this->applyGlobalPermission('accountUpgrades', 'allowed', 'avatar', 'allowed');

			$applied = true;
		}

		return $applied;
	}

	/**
	 * @return array
	 */
	protected function getProviders()
	{
		$providers = [
			'yoomoney' => 'YooMoney',
			'freekassa' => 'FreeKassa',
			'qiwiwallet' => 'QiwiWallet',
			'webmoney' => 'WebMoney',
			'interkassa' => 'InterKassa',
			'robokassa' => 'RoboKassa'
		];

		return $providers;
	}
}